{
  This file was automatically generated on 2012-09-26.

  If you have a bugfix for this file and want to commit it,
  please fix the bug in the generator. You can find a link
  to the generator git on tinkerforge.com
}

unit BrickletDualRelay;

{$ifdef FPC}{$mode OBJFPC}{$H+}{$endif}

interface

uses
  Device, IPConnection, LEConverter;

const
  BRICKLET_DUAL_RELAY_FUNCTION_SET_STATE = 1;
  BRICKLET_DUAL_RELAY_FUNCTION_GET_STATE = 2;
  BRICKLET_DUAL_RELAY_FUNCTION_SET_MONOFLOP = 3;
  BRICKLET_DUAL_RELAY_FUNCTION_GET_MONOFLOP = 4;
  BRICKLET_DUAL_RELAY_CALLBACK_MONOFLOP_DONE = 5;

type
  TBrickletDualRelayNotifyMonoflopDone = procedure(const relay: byte; const state: boolean) of object;

  /// <summary>
  ///  Device for controlling two relays
  /// </summary>
  TBrickletDualRelay = class(TDevice)
  private
    monoflopDoneCallback: TBrickletDualRelayNotifyMonoflopDone;
  protected
    procedure CallbackWrapperMonoflopDone(const packet: TByteArray); virtual;
  public
    /// <summary>
    ///  Creates an object with the unique device ID <c>uid</c>. This object can
    ///  then be added to the IP connection.
    /// </summary>
    constructor Create(const uid_: string);

    /// <summary>
    ///  Sets the state of the relays, *true* means on and *false* means off. 
    ///  For example: (true, false) turns relay 1 on and relay 2 off.
    ///  
    ///  If you just want to set one of the relays and don't know the current state
    ///  of the other relay, you can get the state with <see cref="BrickletDualRelay.TBrickletDualRelay.GetState"/>.
    ///  
    ///  Running monoflop timers will be overwritten if this function is called.
    ///  
    ///  The default value is (false, false).
    /// </summary>
    procedure SetState(const relay1: boolean; const relay2: boolean); virtual;

    /// <summary>
    ///  Returns the state of the relays, *true* means on and *false* means off.
    /// </summary>
    procedure GetState(out relay1: boolean; out relay2: boolean); virtual;

    /// <summary>
    ///  The first parameter can be 1 or 2 (relay 1 or relay 2). The second parameter 
    ///  is the desired state of the relay (*true* means on and *false* means off).
    ///  The third parameter indicates the time (in ms) that the relay should hold 
    ///  the state.
    ///  
    ///  If this function is called with the parameters (1, true, 1500):
    ///  Relay 1 will turn on and in 1.5s it will turn off again.
    ///  
    ///  A monoflop can be used as a failsafe mechanism. For example: Lets assume you 
    ///  have a RS485 bus and a Dual Relay Bricklet connected to one of the slave 
    ///  stacks. You can now call this function every second, with a time parameter
    ///  of two seconds. The relay will be on all the time. If now the RS485 
    ///  connection is lost, the relay will turn off in at most two seconds.
    ///  
    ///  .. versionadded:: 1.1.1
    /// </summary>
    procedure SetMonoflop(const relay: byte; const state: boolean; const time: longword); virtual;

    /// <summary>
    ///  Returns (for the given relay) the current state and the time as set by 
    ///  <see cref="BrickletDualRelay.TBrickletDualRelay.SetMonoflop"/> as well as the remaining time until the state flips.
    ///  
    ///  If the timer is not running currently, the remaining time will be returned
    ///  as 0.
    ///  
    ///  .. versionadded:: 1.1.1
    /// </summary>
    procedure GetMonoflop(const relay: byte; out state: boolean; out time: longword; out timeRemaining: longword); virtual;

    /// <summary>
    ///  This callback is triggered whenever a monoflop timer reaches 0. The 
    ///  parameter contain the relay (1 or 2) and the current state of the relay 
    ///  (the state after the monoflop).
    ///  
    ///  .. versionadded:: 1.1.1
    /// </summary>
    property OnMonoflopDone: TBrickletDualRelayNotifyMonoflopDone read monoflopDoneCallback write monoflopDoneCallback;
  end;

implementation

constructor TBrickletDualRelay.Create(const uid_: string);
begin
  inherited Create(uid_);
  expectedName := 'Dual Relay Bricklet';
  bindingVersion[0] := 1;
  bindingVersion[1] := 0;
  bindingVersion[2] := 1;
  callbackWrappers[BRICKLET_DUAL_RELAY_CALLBACK_MONOFLOP_DONE] := {$ifdef FPC}@{$endif}CallbackWrapperMonoflopDone;
end;

procedure TBrickletDualRelay.SetState(const relay1: boolean; const relay2: boolean);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_DUAL_RELAY_FUNCTION_SET_STATE, 6);
  LEConvertBooleanTo(relay1, 4, request);
  LEConvertBooleanTo(relay2, 5, request);
  SendRequestNoResponse(request);
end;

procedure TBrickletDualRelay.GetState(out relay1: boolean; out relay2: boolean);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_DUAL_RELAY_FUNCTION_GET_STATE, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_DUAL_RELAY_FUNCTION_GET_STATE, response);
  relay1 := LEConvertBooleanFrom(4, response);
  relay2 := LEConvertBooleanFrom(5, response);
end;

procedure TBrickletDualRelay.SetMonoflop(const relay: byte; const state: boolean; const time: longword);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_DUAL_RELAY_FUNCTION_SET_MONOFLOP, 10);
  LEConvertUInt8To(relay, 4, request);
  LEConvertBooleanTo(state, 5, request);
  LEConvertUInt32To(time, 6, request);
  SendRequestNoResponse(request);
end;

procedure TBrickletDualRelay.GetMonoflop(const relay: byte; out state: boolean; out time: longword; out timeRemaining: longword);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_DUAL_RELAY_FUNCTION_GET_MONOFLOP, 5);
  LEConvertUInt8To(relay, 4, request);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_DUAL_RELAY_FUNCTION_GET_MONOFLOP, response);
  state := LEConvertBooleanFrom(4, response);
  time := LEConvertUInt32From(5, response);
  timeRemaining := LEConvertUInt32From(9, response);
end;

procedure TBrickletDualRelay.CallbackWrapperMonoflopDone(const packet: TByteArray);
var relay: byte; state: boolean;
begin
  if (Assigned(monoflopDoneCallback)) then begin
    relay := LEConvertUInt8From(4, packet);
    state := LEConvertBooleanFrom(5, packet);
    monoflopDoneCallback(relay, state);
  end;
end;

end.
